@@ -15,6 +15,7 @@ from functools import partial
 from StringIO import StringIO
 import sys
 
+from nose import SkipTest
 from nose.tools import eq_
 
 # This tests that __all__ is correct, since we use below everything that should
@@ -101,6 +102,25 @@ def test_horizontal_location():
                              unicode_cap('rc'))
 
 
+def test_null_location():
+    """Make sure ``location()`` with no args just does position restoration."""
+    t = TestTerminal(stream=StringIO(), force_styling=True)
+    with t.location():
+        pass
+    eq_(t.stream.getvalue(), unicode_cap('sc') +
+                             unicode_cap('rc'))
+
+
+def test_zero_location():
+    """Make sure ``location()`` pays attention to 0-valued args."""
+    t = TestTerminal(stream=StringIO(), force_styling=True)
+    with t.location(0, 0):
+        pass
+    eq_(t.stream.getvalue(), unicode_cap('sc') +
+                             unicode_parm('cup', 0, 0) +
+                             unicode_cap('rc'))
+
+
 def test_null_fileno():
     """Make sure ``Terminal`` works when ``fileno`` is ``None``.
 
@@ -134,6 +154,46 @@ def test_mnemonic_colors():
     eq_(t.on_bright_green, on_color(10))
 
 
+def test_callable_numeric_colors():
+    """``color(n)`` should return a formatting wrapper."""
+    t = TestTerminal()
+    eq_(t.color(5)('smoo'), t.magenta + 'smoo' + t.normal)
+    eq_(t.color(5)('smoo'), t.color(5) + 'smoo' + t.normal)
+    eq_(t.on_color(2)('smoo'), t.on_green + 'smoo' + t.normal)
+    eq_(t.on_color(2)('smoo'), t.on_color(2) + 'smoo' + t.normal)
+
+
+def test_null_callable_numeric_colors():
+    """``color(n)`` should be a no-op on null terminals."""
+    t = TestTerminal(stream=StringIO())
+    eq_(t.color(5)('smoo'), 'smoo')
+    eq_(t.on_color(6)('smoo'), 'smoo')
+
+
+def test_naked_color_cap():
+    """``term.color`` should return a stringlike capability."""
+    t = TestTerminal()
+    eq_(t.color + '', t.setaf + '')
+
+
+def test_number_of_colors_without_tty():
+    """``number_of_colors`` should return 0 when there's no tty."""
+    # Hypothesis: once setupterm() has run and decided the tty supports 256
+    # colors, it never changes its mind.
+    raise SkipTest
+
+    t = TestTerminal(stream=StringIO())
+    eq_(t.number_of_colors, 0)
+    t = TestTerminal(stream=StringIO(), force_styling=True)
+    eq_(t.number_of_colors, 0)
+
+
+def test_number_of_colors_with_tty():
+    """``number_of_colors`` should work."""
+    t = TestTerminal()
+    eq_(t.number_of_colors, 256)
+
+
 def test_formatting_functions():
     """Test crazy-ass formatting wrappers, both simple and compound."""
     t = TestTerminal()
@@ -182,3 +242,9 @@ def test_nice_formatting_errors():
         t.bold_misspelled('a', 'b')  # >1 string arg
     except TypeError, e:
         assert 'probably misspelled' not in e.args[0]
+
+
+def test_init_descriptor_always_initted():
+    """We should be able to get a height and width even on no-tty Terminals."""
+    t = Terminal(stream=StringIO())
+    eq_(type(t.height), int)
